/*
 *  serialBuffer.c
 *  
 *
 *  Created by Control Plus on Mon Aug 15 2005.
 *
 */

#include "serialBuffer.h"
/* Converts a numeric baud rate to a POSIX speed_t. */
//--------------------------------------------------------------------------------------------------

speed_t baud_to_speed(int baud)
{
  switch (baud)
    {
    case 0:
      return B0;
    case 50:
      return B50;
    case 75:
      return B75;
    case 110:
      return B110;
    case 134:
      return B134;
    case 150:
      return B150;
    case 200:
      return B200;
    case 300:
      return B300;
    case 600:
      return B600;
    case 1200:
      return B1200;
    case 1800:
      return B1800;
    case 2400:
      return B2400;
    case 4800:
      return B4800;
    case 9600:
      return B9600;

#ifdef B19200
    case 19200:
      return B19200;
#else /* B19200 */
#ifdef EXTA
    case 19200:
      return EXTA;
#endif /* EXTA */
#endif /* B19200 */

#ifdef B38400
    case 38400:
      return B38400;
#else /* B38400 */
#ifdef EXTB
    case 38400:
      return EXTB;
#endif /* EXTB */
#endif /* B38400 */

#ifdef B7200
    case 7200:
      return B7200;
#endif /* B7200 */
#ifdef B14400
    case 14400:
      return B14400;
#endif /* B14400 */
#ifdef B28800
    case 28800:
      return B28800;
#endif /* B28800 */
#ifdef B57600
    case 57600:
      return B57600;
#endif /* B57600 */
#ifdef B76800
    case 76800:
      return B76800;
#endif /* B76800 */
#ifdef B115200
    case 115200:
      return B115200;
#endif /* B115200 */
#ifdef B230400
    case 230400:
      return B230400;
#endif /* B230400 */
    default:
      return B9600;
    }
}
//--------------------------------------------------------------------------------------------------
int OpenPort(const char *name,int baud,int timeout)
{
    int i;
    int 		fileDescriptor = -1;
    int 		handshake;
    struct termios	options;
	int errno;
    
    // Open the serial port read/write, with no controlling terminal, and don't wait for a connection.
    // The O_NONBLOCK flag also causes subsequent I/O on the device to be non-blocking.
    // See open(2) ("man 2 open") for details.
    
    printf("openning port %s\n",name);
    
    fileDescriptor = open(name, O_RDWR | O_NOCTTY | O_NONBLOCK);
    if (fileDescriptor == -1)
    {
        printf("Error opening serial port %s - %s(%d).\n",name, strerror(errno), errno);
        goto error;
    }

    // Note that open() follows POSIX semantics: multiple open() calls to the same file will succeed
    // unless the TIOCEXCL ioctl is issued. This will prevent additional opens except by root-owned
    // processes.
    // See tty(4) ("man 4 tty") and ioctl(2) ("man 2 ioctl") for details.
    
    if (ioctl(fileDescriptor, TIOCEXCL) == -1)
    {
        printf("Error setting TIOCEXCL on %s - %s(%d).\n",name, strerror(errno), errno);
        goto error;
    }
    
    // Now that the device is open, clear the O_NONBLOCK flag so subsequent I/O will block.
    // See fcntl(2) ("man 2 fcntl") for details.
    
    if (fcntl(fileDescriptor, F_SETFL, 0) == -1)
    {
        printf("Error clearing O_NONBLOCK %s - %s(%d).\n",name, strerror(errno), errno);
        goto error;
    }
 
//************
//MAKE SURE THE TERMIOS IS EMPTY
   
    /*options.c_iflag = 0;
    options.c_oflag = 0;
    options.c_cflag = 0;
    options.c_lflag = 0;
    for(i=0;i<NCCS;i++)
        options.c_cc[i]=0;*/

    
    // Set raw input (non-canonical) mode, with reads blocking until either a single character 
    // has been received or a three second timeout expires.
    // See tcsetattr(4) ("man 4 tcsetattr") and termios(4) ("man 4 termios") for details.
	
	bzero(&options, sizeof(options));
	
    cfmakeraw(&options);
    options.c_cc[VMIN] = 0;// when this is set to 1 there must be at least 1 byte rxed before the timer starts. thus it will never timeout if the external device is not powered
    options.c_cc[VTIME] = timeout;
        
    // The baud rate, word length, and handshake options can be set as follows:
    
    cfsetspeed(&options, baud_to_speed(baud)); 	// Set baud    
    options.c_cflag |= ( CS8 | CREAD); 	   //| 	// Use 8 bit words and enable reads
    
    // Cause the new options to take effect immediately.
    if (tcsetattr(fileDescriptor, TCSANOW, &options) == -1)
    {
        printf("Error setting tty attributes %s - %s(%d).\n",name, strerror(errno), errno);
        goto error;
    }

    
    // Success
    //portDescriptor = fileDescriptor;
    //PortOpenOK = YES;
    return fileDescriptor;
    
    // Failure path
error:
    if (fileDescriptor != -1)
    {
        close(fileDescriptor);
    }
    else
    {
        //close(portDescriptor);
    }
    
    return -1;
}



//--------------------------------------------------------------------------------------
serialBuffer *newSerialBuffer(int port)
{
    int i;
    serialBuffer *theSB = malloc(sizeof(serialBuffer));
    if(theSB)
    {
            theSB->port = port;
            theSB->inputBufferPointer = 0;
            theSB->inputBufferSize = 0;
            for(i=0;i<kMaxBufferSize;i++)
                theSB->inputBuffer[i] = 0;
    }
    return theSB;
}
//--------------------------------------------------------------------------------------
int getOneByte(Byte *data, serialBuffer *SB)
{
    int response_len = 0;
    
    
    if(SB->inputBufferPointer>=SB->inputBufferSize)
    {
        response_len = read(SB->port, SB->inputBuffer, kMaxBufferSize);
        if(response_len>=0)
        {
            //printf("outlet %d - response_len = %d - error = %d port = %d\n",PM->outlet,response_len,errno,PM->port);
            if(response_len==0) 
            {
                    //printf("OUTLET %d timeout on RX\n",PM->outlet);
                    return NO; // we have had a timeout
            }
            SB->inputBufferSize = response_len;
            SB->inputBufferPointer = 0;
        }
        else
        {
            printf("SerialBuffer - read data error\n");
        }
    }
    if((SB->inputBufferPointer<SB->inputBufferSize)&&(response_len>=0))
    {
        data[0] = SB->inputBuffer[SB->inputBufferPointer++];
        //printf("%.2X\n",data[0]);
        return YES;
    }
    return NO;

}
